#ifndef LINKEDLISTH
#define LINKEDLISTH

/* LinkedList.h -- declarations for singly-linked list

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

Modification History:

*/

#include "SeqCltn.h"
#include "Link.h"

extern Class class_LinkedList;

class LinkedList : public SeqCltn {
	Link* firstLink;	// pointer to first Link of list 
	Link* lastLink;		// pointer to last Link of list 
	UNSIGNED count;		// count of items on list 
	void errDblLnk(const char* fn, const Link& lnk);
	void errEmpty(const char* fn);
	void errNotFound(const char* fn, const Object& ob);
	void IndexRangeErr();
public:
	LinkedList();
	LinkedList(fileDescTy&,LinkedList&);
	LinkedList(istream&,LinkedList&);
	bool operator!=(const LinkedList& a)	{ return !(*this==a); }
	bool operator==(const LinkedList&);
	virtual obid add(const Object&);
	virtual Collection& addContentsTo(Collection& cltn);
	virtual obid addFirst(const Object& ob);
	virtual obid addLast(const Object& ob);
	virtual obid& at(int i);
	virtual void atAllPut(const Object& ob);	// should not implement 
	virtual void deepenShallowCopy();
	virtual obid doNext(DoCltnPos&);
	virtual obid first();
	virtual UNSIGNED hash();
 	virtual int indexOf(const Object& ob);
	virtual int indexOfSubCollection(const SeqCltn& cltn, int start=0);	// shouldNotImplement 
	virtual const Class* isA();
	virtual bool isEmpty();
	virtual bool isEqual(const Object&);
	virtual obid last();
	virtual UNSIGNED occurrencesOf(const Object&);
	virtual void printOn(ostream& strm);
	virtual obid remove(const Object&);
	virtual obid removeFirst();
	virtual obid removeLast();
	virtual void replaceFrom(int start, int stop, const SeqCltn& replacement, int startAt =0);	// shouldNotImplement 
	virtual void reSize(UNSIGNED newSize);
	virtual UNSIGNED size();
	virtual const Class* species();
	virtual void storer(fileDescTy&);
	virtual void storer(ostream&);
};

#endif
